#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/8/4 17:22
Desc: 历年世界 500 强榜单数据
https://www.fortunechina.com/fortune500/index.htm
特殊情况说明：
2010年由于网页端没有公布公司所属的国家, 故 2010 年数据没有国家这列
"""

from functools import lru_cache
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm


@lru_cache()
def _fortune_rank_year_url_map() -> dict:
    """
    年份和网址映射
    https://www.fortunechina.com/fortune500/index.htm
    :return: 年份和网址映射
    :rtype: dict
    """
    url = "https://www.fortunechina.com/fortune500/index.htm"
    r = requests.get(url)
    soup = BeautifulSoup(r.text, features="lxml")
    url_2023 = "https://www.fortunechina.com/fortune500/c/2023-08/02/content_436874.htm"
    node_list = soup.find_all(name="div", attrs={"class": "swiper-slide"})
    url_list = [item.find("a")["href"] for item in node_list]
    year_list = [item.find("a").text for item in node_list]
    year_url_map = dict(zip(year_list, url_list))
    year_url_map["2023"] = url_2023
    return year_url_map


def fortune_rank(year: str = "2015") -> pd.DataFrame:
    """
    财富 500 强公司从 1996 年开始的排行榜
    https://www.fortunechina.com/fortune500/index.htm
    :param year: str 年份
    :return: pandas.DataFrame
    """
    year_url_map = _fortune_rank_year_url_map()
    url = year_url_map[year]
    r = requests.get(url)
    r.encoding = "utf-8"
    if int(year) < 2007:
        df = pd.read_html(StringIO(r.text))[0].iloc[1:-1,]
        df.columns = pd.read_html(StringIO(r.text))[0].iloc[0, :].tolist()
        return df
    elif 2006 < int(year) < 2010:
        df = pd.read_html(StringIO(r.text))[0].iloc[1:,]
        df.columns = pd.read_html(StringIO(r.text))[0].iloc[0, :].tolist()
        for page in tqdm(range(2, 11), leave=False):
            # page =2
            r = requests.get(url.rsplit(".", maxsplit=1)[0] + "_" + str(page) + ".htm")
            r.encoding = "utf-8"
            temp_df = pd.read_html(StringIO(r.text))[0].iloc[1:,]
            temp_df.columns = pd.read_html(StringIO(r.text))[0].iloc[0, :].tolist()
            df = pd.concat(objs=[df, temp_df], ignore_index=True)
        return df
    else:
        df = pd.read_html(StringIO(r.text))[0]
        return df


if __name__ == "__main__":
    fortune_rank_df = fortune_rank(year="2023")  # 2010 不一样
    print(fortune_rank_df)

    fortune_rank_df = fortune_rank(year="2022")  # 2010 不一样
    print(fortune_rank_df)

    fortune_rank_df = fortune_rank(year="2008")  # 2010 不一样
    print(fortune_rank_df)

    fortune_rank_df = fortune_rank(year="2008")  # 2010 不一样
    print(fortune_rank_df)

    fortune_rank_df = fortune_rank(year="2009")  # 2010 不一样
    print(fortune_rank_df)

    for item in range(1996, 2008):
        print(item)
        fortune_rank_df = fortune_rank(year=str(item))  # 2010 不一样
        print(fortune_rank_df)

    for item in range(2010, 2023):
        print(item)
        fortune_rank_df = fortune_rank(year=str(item))  # 2010 不一样
        print(fortune_rank_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/10/1 22:00
Desc: 东方财富网-数据中心-特色数据-股东户数
https://data.eastmoney.com/gdhs/
"""

import pandas as pd
import requests

from akshare.utils.tqdm import get_tqdm


def stock_zh_a_gdhs(symbol: str = "20230930") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股东户数
    https://data.eastmoney.com/gdhs/
    :param symbol: choice of {"最新", "每个季度末"}, 其中 每个季度末需要写成 `20230930` 格式
    :type symbol: str
    :return: 股东户数
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    if symbol == "最新":
        params = {
            "sortColumns": "HOLD_NOTICE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": "500",
            "pageNumber": "1",
            "reportName": "RPT_HOLDERNUMLATEST",
            "columns": "SECURITY_CODE,SECURITY_NAME_ABBR,END_DATE,INTERVAL_CHRATE,AVG_MARKET_CAP,AVG_HOLD_NUM,"
            "TOTAL_MARKET_CAP,TOTAL_A_SHARES,HOLD_NOTICE_DATE,HOLDER_NUM,PRE_HOLDER_NUM,"
            "HOLDER_NUM_CHANGE,HOLDER_NUM_RATIO,END_DATE,PRE_END_DATE",
            "quoteColumns": "f2,f3",
            "source": "WEB",
            "client": "WEB",
        }
    else:
        params = {
            "sortColumns": "HOLD_NOTICE_DATE,SECURITY_CODE",
            "sortTypes": "-1,-1",
            "pageSize": "500",
            "pageNumber": "1",
            "reportName": "RPT_HOLDERNUM_DET",
            "columns": "SECURITY_CODE,SECURITY_NAME_ABBR,END_DATE,INTERVAL_CHRATE,AVG_MARKET_CAP,"
            "AVG_HOLD_NUM,TOTAL_MARKET_CAP,TOTAL_A_SHARES,HOLD_NOTICE_DATE,HOLDER_NUM,"
            "PRE_HOLDER_NUM,HOLDER_NUM_CHANGE,HOLDER_NUM_RATIO,END_DATE,PRE_END_DATE",
            "quoteColumns": "f2,f3",
            "source": "WEB",
            "client": "WEB",
            "filter": f"(END_DATE='{symbol[:4] + '-' + symbol[4:6] + '-' + symbol[6:]}')",
        }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page_num in tqdm(range(1, total_page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page_num,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "代码",
        "名称",
        "股东户数统计截止日-本次",
        "区间涨跌幅",
        "户均持股市值",
        "户均持股数量",
        "总市值",
        "总股本",
        "公告日期",
        "股东户数-本次",
        "股东户数-上次",
        "股东户数-增减",
        "股东户数-增减比例",
        "股东户数统计截止日-上次",
        "最新价",
        "涨跌幅",
    ]
    big_df = big_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌幅",
            "股东户数-本次",
            "股东户数-上次",
            "股东户数-增减",
            "股东户数-增减比例",
            "区间涨跌幅",
            "股东户数统计截止日-本次",
            "股东户数统计截止日-上次",
            "户均持股市值",
            "户均持股数量",
            "总市值",
            "总股本",
            "公告日期",
        ]
    ]
    big_df["最新价"] = pd.to_numeric(big_df["最新价"], errors="coerce")
    big_df["涨跌幅"] = pd.to_numeric(big_df["涨跌幅"], errors="coerce")
    big_df["股东户数-本次"] = pd.to_numeric(big_df["股东户数-本次"], errors="coerce")
    big_df["股东户数-上次"] = pd.to_numeric(big_df["股东户数-上次"], errors="coerce")
    big_df["股东户数-增减"] = pd.to_numeric(big_df["股东户数-增减"], errors="coerce")
    big_df["股东户数-增减比例"] = pd.to_numeric(
        big_df["股东户数-增减比例"], errors="coerce"
    )
    big_df["区间涨跌幅"] = pd.to_numeric(big_df["区间涨跌幅"], errors="coerce")
    big_df["股东户数统计截止日-本次"] = pd.to_datetime(
        big_df["股东户数统计截止日-本次"], errors="coerce"
    ).dt.date
    big_df["股东户数统计截止日-上次"] = pd.to_datetime(
        big_df["股东户数统计截止日-上次"], errors="coerce"
    ).dt.date
    big_df["户均持股市值"] = pd.to_numeric(big_df["户均持股市值"], errors="coerce")
    big_df["户均持股数量"] = pd.to_numeric(big_df["户均持股数量"], errors="coerce")
    big_df["总市值"] = pd.to_numeric(big_df["总市值"], errors="coerce")
    big_df["总股本"] = pd.to_numeric(big_df["总股本"], errors="coerce")
    big_df["公告日期"] = pd.to_datetime(big_df["公告日期"], errors="coerce").dt.date
    return big_df


def stock_zh_a_gdhs_detail_em(symbol: str = "000001") -> pd.DataFrame:
    """
    东方财富网-数据中心-特色数据-股东户数详情
    https://data.eastmoney.com/gdhs/detail/000002.html
    :param symbol: 股票代码
    :type symbol: str
    :return: 股东户数
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "sortColumns": "END_DATE",
        "sortTypes": "-1",
        "pageSize": "500",
        "pageNumber": "1",
        "reportName": "RPT_HOLDERNUM_DET",
        "columns": "SECURITY_CODE,SECURITY_NAME_ABBR,CHANGE_SHARES,CHANGE_REASON,END_DATE,INTERVAL_CHRATE,"
        "AVG_MARKET_CAP,AVG_HOLD_NUM,TOTAL_MARKET_CAP,TOTAL_A_SHARES,HOLD_NOTICE_DATE,HOLDER_NUM,"
        "PRE_HOLDER_NUM,HOLDER_NUM_CHANGE,HOLDER_NUM_RATIO,END_DATE,PRE_END_DATE",
        "quoteColumns": "f2,f3",
        "filter": f'(SECURITY_CODE="{symbol}")',
        "source": "WEB",
        "client": "WEB",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    total_page_num = data_json["result"]["pages"]
    big_df = pd.DataFrame()
    tqdm = get_tqdm()
    for page_num in tqdm(range(1, total_page_num + 1), leave=False):
        params.update(
            {
                "pageNumber": page_num,
            }
        )
        r = requests.get(url, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(data_json["result"]["data"])
        big_df = pd.concat(objs=[big_df, temp_df], ignore_index=True)
    big_df.columns = [
        "代码",
        "名称",
        "股本变动",
        "股本变动原因",
        "股东户数统计截止日",
        "区间涨跌幅",
        "户均持股市值",
        "户均持股数量",
        "总市值",
        "总股本",
        "股东户数公告日期",
        "股东户数-本次",
        "股东户数-上次",
        "股东户数-增减",
        "股东户数-增减比例",
        "-",
        "-",
        "-",
    ]
    big_df = big_df[
        [
            "股东户数统计截止日",
            "区间涨跌幅",
            "股东户数-本次",
            "股东户数-上次",
            "股东户数-增减",
            "股东户数-增减比例",
            "户均持股市值",
            "户均持股数量",
            "总市值",
            "总股本",
            "股本变动",
            "股本变动原因",
            "股东户数公告日期",
            "代码",
            "名称",
        ]
    ]
    big_df["区间涨跌幅"] = pd.to_numeric(big_df["区间涨跌幅"], errors="coerce")
    big_df["股东户数-本次"] = pd.to_numeric(big_df["股东户数-本次"], errors="coerce")
    big_df["股东户数-上次"] = pd.to_numeric(big_df["股东户数-上次"], errors="coerce")
    big_df["股东户数-增减"] = pd.to_numeric(big_df["股东户数-增减"], errors="coerce")
    big_df["股东户数-增减比例"] = pd.to_numeric(
        big_df["股东户数-增减比例"], errors="coerce"
    )
    big_df["户均持股市值"] = pd.to_numeric(big_df["户均持股市值"], errors="coerce")
    big_df["户均持股数量"] = pd.to_numeric(big_df["户均持股数量"], errors="coerce")
    big_df["总市值"] = pd.to_numeric(big_df["总市值"], errors="coerce")
    big_df["总股本"] = pd.to_numeric(big_df["总股本"], errors="coerce")
    big_df["股本变动"] = pd.to_numeric(big_df["股本变动"], errors="coerce")
    big_df["股东户数统计截止日"] = pd.to_datetime(
        big_df["股东户数统计截止日"], errors="coerce"
    ).dt.date
    big_df["股东户数公告日期"] = pd.to_datetime(
        big_df["股东户数公告日期"], errors="coerce"
    ).dt.date
    big_df.sort_values(by=['股东户数统计截止日'], ignore_index=True, inplace=True)
    return big_df


if __name__ == "__main__":
    stock_zh_a_gdhs_df = stock_zh_a_gdhs(symbol="20230930")
    print(stock_zh_a_gdhs_df)

    stock_zh_a_gdhs_detail_em_df = stock_zh_a_gdhs_detail_em(symbol="002631")
    print(stock_zh_a_gdhs_detail_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/9/9 14:57
Desc: 中国外汇交易中心暨全国银行间同业拆借中心-基准-外汇市场-外汇掉期曲线-外汇掉漆 C-Swap 定盘曲线
https://www.chinamoney.org.cn/chinese/bkcurvfsw
"""

import ssl

import pandas as pd
import requests
from requests.adapters import HTTPAdapter


class LegacySSLAdapter(HTTPAdapter):
    def init_poolmanager(self, *args, **kwargs):
        context = ssl.create_default_context()
        # 允许不安全的 legacy renegotiation
        context.options |= ssl.OP_LEGACY_SERVER_CONNECT
        kwargs['ssl_context'] = context
        return super().init_poolmanager(*args, **kwargs)


def fx_c_swap_cm():
    """
    中国外汇交易中心暨全国银行间同业拆借中心-基准-外汇市场-外汇掉期曲线-外汇掉期 C-Swap 定盘曲线
    https://www.chinamoney.org.cn/chinese/bkcurvfsw
    :return: 外汇掉期 C-Swap 定盘曲线
    :rtype: pandas.DataFrame
    """
    session = requests.Session()
    session.mount(prefix='https://', adapter=LegacySSLAdapter())
    url = "https://www.chinamoney.org.cn/r/cms/www/chinamoney/data/fx/fx-c-sw-curv-USD.CNY.json"
    payload = {
        "t": "1757402201554",
    }
    r = session.post(url, data=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['records'])
    temp_df.rename(columns={
        "curveTime": "日期时间",
        "tenor": "期限品种",
        "swapPnt": "掉期点(Pips)",
        "dataSource": "掉期点数据源",
        "swapAllPrc": "全价汇率",
    }, inplace=True)
    temp_df = temp_df[[
        "日期时间",
        "期限品种",
        "掉期点(Pips)",
        "掉期点数据源",
        "全价汇率",
    ]]
    temp_df["掉期点(Pips)"] = pd.to_numeric(temp_df["掉期点(Pips)"], errors='coerce')
    temp_df["全价汇率"] = pd.to_numeric(temp_df["全价汇率"], errors='coerce')
    return temp_df


if __name__ == '__main__':
    fx_c_swap_cm_df = fx_c_swap_cm()
    print(fx_c_swap_cm_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/3 17:08
Desc: 金十数据中心-经济指标-欧元区
金十数据中心-经济指标-欧元区-国民经济运行状况-经济状况
金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平
金十数据中心-经济指标-欧元区-国民经济运行状况-劳动力市场
金十数据中心-经济指标-欧元区-贸易状况
金十数据中心-经济指标-欧元区-产业指标
金十数据中心-经济指标-欧元区-领先指标
"""

import time

import pandas as pd
import requests
from tqdm import tqdm


# 金十数据中心-经济指标-欧元区-国民经济运行状况
# 金十数据中心-经济指标-欧元区-国民经济运行状况-经济状况
# 金十数据中心-经济指标-欧元区-国民经济运行状况-经济状况-欧元区季度GDP年率报告
def macro_euro_gdp_yoy() -> pd.DataFrame:
    """
    欧元区季度 GDP 年率报告, 数据区间从 20131114-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_gdp_yoy
    :return: 欧元区季度 GDP 年率报告
    :rtype: pandas.DataFrame
    """
    ec = 84
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {"category": "ec", "attr_id": ec}
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)

    big_df["商品"] = "欧元区季度GDP年率"

    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区CPI月率报告
def macro_euro_cpi_mom() -> pd.DataFrame:
    """
    欧元区 CPI 月率报告, 数据区间从 19900301-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_cpi_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_cpi_mom_all.js?v=1578578318
    :return: 欧元区CPI月率报告
    :rtype: pandas.Series
    """
    ec = 84
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {"category": "ec", "attr_id": ec}
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区CPI月率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区CPI年率报告
def macro_euro_cpi_yoy() -> pd.DataFrame:
    """
    欧元区CPI年率报告, 数据区间从19910201-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_cpi_yoy
    https://cdn.jin10.com/dc/reports/dc_eurozone_cpi_yoy_all.js?v=1578578404
    :return: 欧元区CPI年率报告-今值(%)
    :rtype: pandas.Series
    """
    ec = 8
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区CPI年率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区PPI月率报告
def macro_euro_ppi_mom() -> pd.DataFrame:
    """
    欧元区PPI月率报告, 数据区间从19810301-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_ppi_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_ppi_mom_all.js?v=1578578493
    :return: 欧元区PPI月率报告-今值(%)
    :rtype: pandas.Series
    """
    ec = 36
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区PPI月率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区零售销售月率报告
def macro_euro_retail_sales_mom() -> pd.DataFrame:
    """
    欧元区零售销售月率报告, 数据区间从20000301-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_retail_sales_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_retail_sales_mom_all.js?v=1578578576
    :return: 欧元区零售销售月率报告-今值(%)
    :rtype: pandas.Series
    """
    ec = 38
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区零售销售月率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-国民经济运行状况-劳动力市场-欧元区季调后就业人数季率报告
def macro_euro_employment_change_qoq() -> pd.DataFrame:
    """
    欧元区季调后就业人数季率报告, 数据区间从20083017-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_employment_change_qoq
    https://cdn.jin10.com/dc/reports/dc_eurozone_employment_change_qoq_all.js?v=1578578699
    :return: 欧元区季调后就业人数季率报告-今值(%)
    :rtype: pandas.Series
    """
    ec = 14
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {"category": "ec", "attr_id": ec}
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区季调后就业人数季率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-国民经济运行状况-劳动力市场-欧元区失业率报告
def macro_euro_unemployment_rate_mom() -> pd.DataFrame:
    """
    欧元区失业率报告, 数据区间从19980501-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_unemployment_rate_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_unemployment_rate_mom_all.js?v=1578578767
    :return: 欧元区失业率报告-今值(%)
    :rtype: pandas.Series
    """
    ec = 46
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区失业率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-贸易状况-欧元区未季调贸易帐报告
def macro_euro_trade_balance() -> pd.DataFrame:
    """
    欧元区未季调贸易帐报告, 数据区间从19990201-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_trade_balance_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_trade_balance_mom_all.js?v=1578577862
    :return: 欧元区未季调贸易帐报告-今值(亿欧元)
    :rtype: pandas.Series
    """
    ec = 43
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区未季调贸易帐"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-贸易状况-欧元区经常帐报告
def macro_euro_current_account_mom() -> pd.DataFrame:
    """
    欧元区经常帐报告, 数据区间从20080221-至今, 前两个值需要去掉
    https://datacenter.jin10.com/reportType/dc_eurozone_current_account_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_current_account_mom_all.js?v=1578577976
    :return: 欧元区经常帐报告-今值(亿欧元)
    :rtype: pandas.Series
    """
    ec = 11
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区经常帐"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-产业指标-欧元区工业产出月率报告
def macro_euro_industrial_production_mom() -> pd.DataFrame:
    """
    欧元区工业产出月率报告, 数据区间从19910301-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_industrial_production_mom
    https://cdn.jin10.com/dc/reports/dc_eurozone_industrial_production_mom_all.js?v=1578577377
    :return: 欧元区工业产出月率报告-今值(%)
    :rtype: pandas.Series
    """
    ec = 19
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区工业产出月率"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-产业指标-欧元区制造业PMI初值报告
def macro_euro_manufacturing_pmi() -> pd.DataFrame:
    """
    欧元区制造业PMI初值报告, 数据区间从20080222-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_manufacturing_pmi
    https://cdn.jin10.com/dc/reports/dc_eurozone_manufacturing_pmi_all.js?v=1578577537
    :return: 欧元区制造业PMI初值报告-今值
    :rtype: pandas.Series
    """
    ec = 30
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区制造业PMI初值"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-产业指标-欧元区服务业PMI终值报告
def macro_euro_services_pmi() -> pd.DataFrame:
    """
    欧元区服务业PMI终值报告, 数据区间从 20080222-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_services_pmi
    https://cdn.jin10.com/dc/reports/dc_eurozone_services_pmi_all.js?v=1578577639
    :return: 欧元区服务业PMI终值报告-今值
    :rtype: pandas.Series
    """
    ec = 41
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }

    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区服务业PMI终值"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-领先指标-欧元区ZEW经济景气指数报告
def macro_euro_zew_economic_sentiment() -> pd.DataFrame:
    """
    欧元区ZEW经济景气指数报告, 数据区间从20080212-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_zew_economic_sentiment
    https://cdn.jin10.com/dc/reports/dc_eurozone_zew_economic_sentiment_all.js?v=1578577013
    :return: 欧元区ZEW经济景气指数报告-今值
    :rtype: pandas.Series
    """
    ec = 48
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区ZEW经济景气指数"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-经济指标-欧元区-领先指标-欧元区Sentix投资者信心指数报告
def macro_euro_sentix_investor_confidence() -> pd.DataFrame:
    """
    欧元区Sentix投资者信心指数报告, 数据区间从20020801-至今
    https://datacenter.jin10.com/reportType/dc_eurozone_sentix_investor_confidence
    https://cdn.jin10.com/dc/reports/dc_eurozone_sentix_investor_confidence_all.js?v=1578577195
    :return: 欧元区Sentix投资者信心指数报告-今值
    :rtype: pandas.Series
    """
    ec = 40
    url = "https://datacenter-api.jin10.com/reports/dates"
    params = {
        "category": "ec",
        "attr_id": ec,
    }
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/107.0.0.0 Safari/537.36",
        "x-app-id": "rU6QIu7JHe2gOUeR",
        "x-csrf-token": "x-csrf-token",
        "x-version": "1.0.0",
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    date_list = data_json["data"]
    date_point_list = [item for num, item in enumerate(date_list) if num % 20 == 0]
    big_df = pd.DataFrame()
    for date in tqdm(date_point_list, leave=False):
        url = "https://datacenter-api.jin10.com/reports/list_v2"
        params = {
            "max_date": f"{date}",
            "category": "ec",
            "attr_id": ec,
        }
        headers = {
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/107.0.0.0 Safari/537.36",
            "x-app-id": "rU6QIu7JHe2gOUeR",
            "x-csrf-token": "x-csrf-token",
            "x-version": "1.0.0",
        }
        r = requests.get(url, headers=headers, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            data_json["data"]["values"],
            columns=[item["name"] for item in data_json["data"]["keys"]],
        )
        big_df = pd.concat([big_df, temp_df], ignore_index=True)
    big_df["商品"] = "欧元区Sentix投资者信心指数"
    big_df = big_df[["商品", "日期", "今值", "预测值", "前值"]]
    big_df["今值"] = pd.to_numeric(big_df["今值"])
    big_df["预测值"] = pd.to_numeric(big_df["预测值"])
    big_df["前值"] = pd.to_numeric(big_df["前值"])
    big_df["日期"] = pd.to_datetime(big_df["日期"]).dt.date
    big_df.sort_values(["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-伦敦金属交易所(LME)-持仓报告
def macro_euro_lme_holding() -> pd.DataFrame:
    """
    伦敦金属交易所(LME)-持仓报告, 数据区间从 20151022-至今
    https://datacenter.jin10.com/reportType/dc_lme_traders_report
    https://cdn.jin10.com/data_center/reports/lme_position.json?_=1591533934658
    :return: 伦敦金属交易所(LME)-持仓报告
    :rtype: pandas.DataFrame
    """
    t = time.time()
    params = {"_": str(int(round(t * 1000)))}
    r = requests.get(
        url="https://cdn.jin10.com/data_center/reports/lme_position.json", params=params
    )
    json_data = r.json()
    temp_df = pd.DataFrame(json_data["values"]).T
    temp_df.fillna(value="[0, 0, 0]", inplace=True)
    big_df = pd.DataFrame()
    for item in temp_df.columns:
        for i in range(3):
            inner_temp_df = temp_df.loc[:, item].apply(lambda x: eval(str(x))[i])
            inner_temp_df.name = inner_temp_df.name + "-" + json_data["keys"][i]["name"]
            big_df = pd.concat(objs=[big_df, inner_temp_df], axis=1)
    big_df = big_df.astype("float")
    big_df = big_df.iloc[:-1, :].copy()
    big_df.reset_index(inplace=True)
    big_df.rename(columns={"index": "日期"}, inplace=True)
    big_df.sort_values(by=["日期"], ignore_index=True, inplace=True)
    return big_df


# 金十数据中心-伦敦金属交易所(LME)-库存报告
def macro_euro_lme_stock() -> pd.DataFrame:
    """
    伦敦金属交易所(LME)-库存报告, 数据区间从 20140702-至今
    https://datacenter.jin10.com/reportType/dc_lme_report
    https://cdn.jin10.com/data_center/reports/lme_stock.json?_=1591535304783
    :return: 伦敦金属交易所(LME)-库存报告
    :rtype: pandas.DataFrame
    """
    t = time.time()
    params = {"_": str(int(round(t * 1000)))}
    r = requests.get(
        url="https://cdn.jin10.com/data_center/reports/lme_stock.json", params=params
    )
    json_data = r.json()
    temp_df = pd.DataFrame(json_data["values"]).T
    big_df = pd.DataFrame()
    for item in temp_df.columns:
        for i in range(3):
            inner_temp_df = temp_df.loc[:, item].apply(lambda x: eval(str(x))[i])
            inner_temp_df.name = inner_temp_df.name + "-" + json_data["keys"][i]["name"]
            big_df = pd.concat(objs=[big_df, inner_temp_df], axis=1)
    big_df.sort_index(inplace=True)
    big_df.reset_index(inplace=True)
    big_df.rename(columns={"index": "日期"}, inplace=True)
    return big_df


if __name__ == "__main__":
    # 金十数据中心-经济指标-欧元区-国民经济运行状况
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-经济状况
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-经济状况-欧元区季度GDP年率报告
    macro_euro_gdp_yoy_df = macro_euro_gdp_yoy()
    print(macro_euro_gdp_yoy_df)

    # 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区CPI月率报告
    macro_euro_cpi_mom_df = macro_euro_cpi_mom()
    print(macro_euro_cpi_mom_df)
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区CPI年率报告
    macro_euro_cpi_yoy_df = macro_euro_cpi_yoy()
    print(macro_euro_cpi_yoy_df)
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区PPI月率报告
    macro_euro_ppi_mom_df = macro_euro_ppi_mom()
    print(macro_euro_ppi_mom_df)
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-物价水平-欧元区零售销售月率报告
    macro_euro_retail_sales_mom_df = macro_euro_retail_sales_mom()
    print(macro_euro_retail_sales_mom_df)

    # 金十数据中心-经济指标-欧元区-国民经济运行状况-劳动力市场
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-劳动力市场-欧元区季调后就业人数季率报告
    macro_euro_employment_change_qoq_df = macro_euro_employment_change_qoq()
    print(macro_euro_employment_change_qoq_df)
    # 金十数据中心-经济指标-欧元区-国民经济运行状况-劳动力市场-欧元区失业率报告
    macro_euro_unemployment_rate_mom_df = macro_euro_unemployment_rate_mom()
    print(macro_euro_unemployment_rate_mom_df)
    # 金十数据中心-经济指标-欧元区-贸易状况
    # 金十数据中心-经济指标-欧元区-贸易状况-欧元区未季调贸易帐报告
    macro_euro_trade_balance_df = macro_euro_trade_balance()
    print(macro_euro_trade_balance_df)
    # 金十数据中心-经济指标-欧元区-贸易状况-欧元区经常帐报告
    macro_euro_current_account_mom_df = macro_euro_current_account_mom()
    print(macro_euro_current_account_mom_df)
    # 金十数据中心-经济指标-欧元区-产业指标
    # 金十数据中心-经济指标-欧元区-产业指标-欧元区工业产出月率报告
    macro_euro_industrial_production_mom_df = macro_euro_industrial_production_mom()
    print(macro_euro_industrial_production_mom_df)
    # 金十数据中心-经济指标-欧元区-产业指标-欧元区制造业PMI初值报告
    macro_euro_manufacturing_pmi_df = macro_euro_manufacturing_pmi()
    print(macro_euro_manufacturing_pmi_df)

    # 金十数据中心-经济指标-欧元区-产业指标-欧元区服务业PMI终值报告
    macro_euro_services_pmi_df = macro_euro_services_pmi()
    print(macro_euro_services_pmi_df)

    # 金十数据中心-经济指标-欧元区-领先指标
    # 金十数据中心-经济指标-欧元区-领先指标-欧元区ZEW经济景气指数报告
    macro_euro_zew_economic_sentiment_df = macro_euro_zew_economic_sentiment()
    print(macro_euro_zew_economic_sentiment_df)

    # 金十数据中心-经济指标-欧元区-领先指标-欧元区Sentix投资者信心指数报告
    macro_euro_sentix_investor_confidence_df = macro_euro_sentix_investor_confidence()
    print(macro_euro_sentix_investor_confidence_df)

    # 金十数据中心-伦敦金属交易所(LME)-持仓报告
    macro_euro_lme_holding_df = macro_euro_lme_holding()
    print(macro_euro_lme_holding_df)

    # 金十数据中心-伦敦金属交易所(LME)-库存报告
    macro_euro_lme_stock_df = macro_euro_lme_stock()
    print(macro_euro_lme_stock_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/12/12 17:00
Desc: 生意社网站采集大宗商品现货价格及相应基差数据, 数据时间段从 20110104-至今
备注：现期差 = 现货价格 - 期货价格(这里的期货价格为结算价)
黄金为 元/克, 白银为 元/千克, 玻璃现货为 元/平方米, 鸡蛋现货为 元/公斤, 鸡蛋期货为 元/500千克, 其余为 元/吨.
焦炭现货规格是: 一级冶金焦; 焦炭期货规格: 介于一级和二级之间, 焦炭现期差仅供参考.
铁矿石现货价格是: 湿吨, 铁矿石期货价格是: 干吨
网页地址: https://www.100ppi.com/sf/
历史数据可以通过修改 url 地址来获取, 比如: https://www.100ppi.com/sf/day-2017-09-12.html
发现生意社的 bugs:
1. 2018-09-12 周三 数据缺失是因为生意社源数据在该交易日缺失: https://www.100ppi.com/sf/day-2018-09-12.html
"""

import datetime
import re
import time
import warnings
from typing import List

import pandas as pd

from akshare.futures import cons
from akshare.futures.requests_fun import pandas_read_html_link
from akshare.futures.symbol_var import chinese_to_english

calendar = cons.get_calendar()


def futures_spot_price_daily(
    start_day: str = "20210201",
    end_day: str = "20210208",
    vars_list: list = cons.contract_symbols,
):
    """
    指定时间段内大宗商品现货价格及相应基差
    https://www.100ppi.com/sf/
    :param start_day: str 开始日期 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象; 默认为当天
    :param end_day: str 结束数据 format：YYYY-MM-DD 或 YYYYMMDD 或 datetime.date对象; 默认为当天
    :param vars_list: list 合约品种如 [RB, AL]; 默认参数为所有商品
    :return: 基差
    :rtype: pandas.DataFrame
    展期收益率数据:
    var               商品品种                      string
    sp                现货价格                      float
    near_symbol       临近交割合约                  string
    near_price        临近交割合约结算价             float
    dom_symbol        主力合约                      string
    dom_price         主力合约结算价                 float
    near_basis        临近交割合约相对现货的基差      float
    dom_basis         主力合约相对现货的基差          float
    near_basis_rate   临近交割合约相对现货的基差率    float
    dom_basis_rate    主力合约相对现货的基差率        float
    date              日期                          string YYYYMMDD
    """
    start_day = (
        cons.convert_date(start_day) if start_day is not None else datetime.date.today()
    )
    end_day = (
        cons.convert_date(end_day)
        if end_day is not None
        else cons.convert_date(cons.get_latest_data_date(datetime.datetime.now()))
    )
    df_list = []
    while start_day <= end_day:
        temp_df = futures_spot_price(start_day, vars_list)
        if temp_df is False:
            return pd.concat(df_list).reset_index(drop=True)
        elif temp_df is not None:
            df_list.append(temp_df)
        start_day += datetime.timedelta(days=1)
    if len(df_list) > 0:
        temp_df = pd.concat(df_list)
        temp_df.reset_index(drop=True, inplace=True)
        return temp_df


def futures_spot_price(
    date: str = "20240430", vars_list: list = cons.contract_symbols
) -> pd.DataFrame:
    """
    指定交易日大宗商品现货价格及相应基差
    https://www.100ppi.com/sf/day-2017-09-12.html
    :param date: 开始日期 format: YYYY-MM-DD 或 YYYYMMDD 或 datetime.date 对象; 为空时为当天
    :param vars_list: 合约品种如 RB、AL 等列表 为空时为所有商品
    :return: pandas.DataFrame
    展期收益率数据:
    var              商品品种                     string
    sp               现货价格                     float
    near_symbol      临近交割合约                  string
    near_price       临近交割合约结算价             float
    dom_symbol       主力合约                     string
    dom_price        主力合约结算价                float
    near_basis       临近交割合约相对现货的基差      float
    dom_basis        主力合约相对现货的基差         float
    near_basis_rate  临近交割合约相对现货的基差率    float
    dom_basis_rate   主力合约相对现货的基差率       float
    date             日期                         string YYYYMMDD
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    if date < datetime.date(2011, 1, 4):
        raise Exception(
            "数据源开始日期为 20110104, 请将获取数据时间点设置在 20110104 后"
        )
    if date.strftime("%Y%m%d") not in calendar:
        warnings.warn(f"{date.strftime('%Y%m%d')}非交易日")
        return pd.DataFrame()
    u1 = "https://www.100ppi.com/sf/"
    u2 = f'https://www.100ppi.com/sf/day-{date.strftime("%Y-%m-%d")}.html'
    i = 1
    while True:
        for url in [u2, u1]:
            try:
                # url = u2
                headers = {
                    "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,"
                              "image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7"
                }
                r = pandas_read_html_link(url, headers=headers)
                string = r[0].loc[1, 1]
                news = "".join(re.findall(r"[0-9]", string))
                if news[3:11] == date.strftime("%Y%m%d"):
                    records = _check_information(r[1], date)
                    records.index = records["symbol"]
                    var_list_in_market = [i for i in vars_list if i in records.index]
                    temp_df = records.loc[var_list_in_market, :]
                    temp_df.reset_index(drop=True, inplace=True)
                    return temp_df
                else:
                    time.sleep(3)
            except Exception as e:  # noqa: E722
                print(
                    f"{date.strftime('%Y-%m-%d')}日生意社数据连接失败[错误信息:{e}]，第{str(i)}次尝试，最多5次"
                )
                i += 1
                if i > 5:
                    print(
                        f"{date.strftime('%Y-%m-%d')}日生意社数据连接失败, 如果当前交易日是 2018-09-12, "
                        f"由于生意社源数据缺失, 无法访问, 否则为重复访问已超过5次，您的地址被网站墙了，"
                        f"请保存好返回数据，稍后从该日期起重试"
                    )
                    return pd.DataFrame()


def _check_information(df_data, date):
    """
    数据验证和计算模块
    :param df_data: pandas.DataFrame 采集的数据
    :param date: datetime.date 具体某一天 YYYYMMDD
    :return: pandas.DataFrame
    中间数据
    symbol  spot_price near_contract  ...  near_basis_rate dom_basis_rate      date
     CU    49620.00        cu1811  ...        -0.002418      -0.003426  20181108
     RB     4551.54        rb1811  ...        -0.013521      -0.134359  20181108
     ZN    22420.00        zn1811  ...        -0.032114      -0.076271  20181108
     AL    13900.00        al1812  ...         0.005396       0.003957  20181108
     AU      274.10        au1811  ...         0.005655       0.020430  20181108
     WR     4806.25        wr1903  ...        -0.180026      -0.237035  20181108
     RU    10438.89        ru1811  ...        -0.020969       0.084406  20181108
     PB    18600.00        pb1811  ...        -0.001344      -0.010215  20181108
     AG     3542.67        ag1811  ...        -0.000754       0.009408  20181108
     BU     4045.53        bu1811  ...        -0.129904      -0.149679  20181108
     HC     4043.33        hc1811  ...        -0.035449      -0.088128  20...
    """
    df_data = df_data.loc[:, [0, 1, 2, 3, 5, 6]]
    df_data.columns = [
        "symbol",
        "spot_price",
        "near_contract",
        "near_contract_price",
        "dominant_contract",
        "dominant_contract_price",
    ]
    records = pd.DataFrame()
    for string in df_data["symbol"].tolist():
        news = "".join(re.findall(r"[\u4e00-\u9fa5]", string))
        if news == "":
            news = string.strip()

        """
        if string == "PTA":
            news = "PTA"
        else:
            news = "".join(re.findall(r"[\u4e00-\u9fa5]", string))
        """

        if news != "" and news not in [
            "商品",
            "价格",
            "上海期货交易所",
            "郑州商品交易所",
            "大连商品交易所",
            "广州期货交易所",
            # 某些天网站没有数据，比如 20180912，此时返回"暂无数据"，但并不是网站被墙了
            "暂无数据",
        ]:
            symbol = chinese_to_english(news)
            record = pd.DataFrame(df_data[df_data["symbol"] == string])
            record.loc[:, "symbol"] = symbol
            record.loc[:, "spot_price"] = record.loc[:, "spot_price"].astype(float)
            if (
                symbol == "JD"
            ):  # 鸡蛋现货为元/公斤, 鸡蛋期货为元/500千克, 其余元/吨(http://www.100ppi.com/sf/)
                record.loc[:, "spot_price"] = float(record["spot_price"].iloc[0]) * 500
            elif (
                symbol == "FG"
            ):  # 上表中现货单位为元/平方米, 期货单位为元/吨. 换算公式：元/平方米*80=元/吨(http://www.100ppi.com/sf/959.html)
                record.loc[:, "spot_price"] = float(record["spot_price"].iloc[0]) * 80
            elif (
                symbol == "LH"
            ):  # 上表中现货单位为元/公斤, 期货单位为元/吨. 换算公式：元/公斤*1000=元/吨(http://www.100ppi.com/sf/959.html)
                record.loc[:, "spot_price"] = float(record["spot_price"].iloc[0]) * 1000
            records = pd.concat([records, record])

    # 20241129:如果某日没有数据，直接返回返回空表
    if records.empty:
        records = df_data.iloc[0:0]
        records["near_basis"] = pd.Series(dtype="float")
        records["dom_basis"] = pd.Series(dtype="float")
        records["near_basis_rate"] = pd.Series(dtype="float")
        records["dom_basis_rate"] = pd.Series(dtype="float")
        records["date"] = pd.Series(dtype="object")
        return records

    records.loc[:, ["near_contract_price", "dominant_contract_price", "spot_price"]] = (
        records.loc[
            :, ["near_contract_price", "dominant_contract_price", "spot_price"]
        ].astype("float")
    )

    records.loc[:, "near_contract"] = records["near_contract"].replace(
        r"[^0-9]*(\d*)$", r"\g<1>", regex=True
    )
    records.loc[:, "dominant_contract"] = records["dominant_contract"].replace(
        r"[^0-9]*(\d*)$", r"\g<1>", regex=True
    )

    records.loc[:, "near_month"] = records.loc[:, "near_contract"]
    records.loc[:, "near_contract"] = records["symbol"] + records.loc[
        :, "near_contract"
    ].astype("int").astype("str")
    records.loc[:, "dominant_month"] = records.loc[:, "dominant_contract"]
    records.loc[:, "dominant_contract"] = records["symbol"] + records.loc[
        :, "dominant_contract"
    ].astype("int").astype("str")

    records["near_contract"] = records["near_contract"].apply(
        lambda x: x.lower()
        if x[:-4]
        in cons.market_exchange_symbols["shfe"] + cons.market_exchange_symbols["dce"]
        else x
    )
    records.loc[:, "dominant_contract"] = records.loc[:, "dominant_contract"].apply(
        lambda x: x.lower()
        if x[:-4]
        in cons.market_exchange_symbols["shfe"] + cons.market_exchange_symbols["dce"]
        else x
    )
    records.loc[:, "near_contract"] = records.loc[:, "near_contract"].apply(
        lambda x: x[:-4] + x[-3:]
        if x[:-4] in cons.market_exchange_symbols["czce"]
        else x
    )
    records.loc[:, "dominant_contract"] = records.loc[:, "dominant_contract"].apply(
        lambda x: x[:-4] + x[-3:]
        if x[:-4] in cons.market_exchange_symbols["czce"]
        else x
    )

    records["near_basis"] = records["near_contract_price"] - records["spot_price"]
    records["dom_basis"] = records["dominant_contract_price"] - records["spot_price"]
    records["near_basis_rate"] = (
        records["near_contract_price"] / records["spot_price"] - 1
    )
    records["dom_basis_rate"] = (
        records["dominant_contract_price"] / records["spot_price"] - 1
    )
    # records.loc[:, "date"] = date.strftime("%Y%m%d")
    records.insert(0, "date", date.strftime("%Y%m%d"))
    return records


def _join_head(content: pd.DataFrame) -> List:
    headers = []
    for s1, s2 in zip(content.iloc[0], content.iloc[1]):
        if s1 != s2:
            s = f"{s1}{s2}"
        else:
            s = s1
        headers.append(s)
    return headers


def futures_spot_price_previous(date: str = "20240430") -> pd.DataFrame:
    """
    具体交易日大宗商品现货价格及相应基差
    https://www.100ppi.com/sf/day-2017-09-12.html
    :param date: 交易日; 历史日期
    :type date: str
    :return: 现货价格及相应基差
    :rtype: pandas.DataFrame
    """
    date = cons.convert_date(date) if date is not None else datetime.date.today()
    if date < datetime.date(2011, 1, 4):
        raise Exception(
            "数据源开始日期为 20110104, 请将获取数据时间点设置在 20110104 后"
        )
    if date.strftime("%Y%m%d") not in calendar:
        warnings.warn(f"{date.strftime('%Y%m%d')}非交易日")
        return pd.DataFrame()
    url = date.strftime("https://www.100ppi.com/sf2/day-%Y-%m-%d.html")
    headers = {
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,"
                  "image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7"
    }
    content = pandas_read_html_link(url, headers=headers)
    main = content[1]
    # Header
    header = _join_head(main)
    # Values
    values = main[main[4].str.endswith("%")]
    values.columns = header
    # Basis
    # 对于没有数据的天，xml文件中没有数据，所以content[2:-1]可能为空
    if len(content[2:-1]) > 0:
        basis = pd.concat(content[2:-1])
    else:
        basis = pd.DataFrame(columns=["主力合约基差", "主力合约基差(%)"])

    basis.columns = ["主力合约基差", "主力合约基差(%)"]
    # 20241125(jasonudu)：因为部分日期，存在多个品种的现货价格，比如20151125的白糖、豆粕、豆油等，如果用商品名来merge，会出现重复列名，所以改用index来merge
    # basis["商品"] = values["商品"].tolist()
    basis.index = values.index
    basis = pd.merge(
        values[["商品", "现货价格", "主力合约代码", "主力合约价格"]],
        basis,
        left_index=True,
        right_index=True,
    )
    basis = pd.merge(
        basis,
        values[
            [
                "180日内主力基差最高",
                "180日内主力基差最低",
                "180日内主力基差平均",
            ]
        ],
        left_index=True,
        right_index=True,
    )
    basis.columns = [
        "商品",
        "现货价格",
        "主力合约代码",
        "主力合约价格",
        "主力合约基差",
        "主力合约变动百分比",
        "180日内主力基差最高",
        "180日内主力基差最低",
        "180日内主力基差平均",
    ]
    basis["主力合约变动百分比"] = basis["主力合约变动百分比"].str.strip("%")
    return basis


if __name__ == "__main__":
    futures_spot_price_daily_df = futures_spot_price_daily(
        start_day="20250708", end_day="20250709", vars_list=["BZ", "RB"]
    )
    print(futures_spot_price_daily_df)

    futures_spot_price_df = futures_spot_price(date="20250620")
    print(futures_spot_price_df)

    futures_spot_price_previous_df = futures_spot_price_previous(date="20240430")
    print(futures_spot_price_previous_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/6/16 18:00
Desc: 唯爱期货-期权保证金
https://www.iweiai.com/qihuo/yuanyou
"""
import requests
import pandas as pd
from io import StringIO

from bs4 import BeautifulSoup
from functools import lru_cache


@lru_cache()
def option_margin_symbol() -> pd.DataFrame:
    """
    获取商品期权品种代码和名称
    :return: 商品期权品种代码和名称
    :rtype: pandas.DataFrame
    """
    url = "https://www.iweiai.com/qiquan/yuanyou"
    r = requests.get(url)
    soup = BeautifulSoup(r.content, features="lxml")
    symbol_text = [item.get_text() for item in soup.find_all("a") if "qiquan" in item['href']]
    symbol_url = [item['href'] for item in soup.find_all("a") if "qiquan" in item['href']]
    symbol_df = pd.DataFrame([symbol_text, symbol_url]).T
    symbol_df.columns = ["symbol", "url"]
    return symbol_df


def option_margin(symbol: str = "原油期权") -> pd.DataFrame:
    """
    获取商品期权保证金
    :param symbol: 商品期权品种名称, 如 "原油期权"，可以通过 ak.option_margin_symbol() 获取所有商品期权品种代码和名称
    :type symbol: str
    :return: 商品期权保证金
    :rtype: pandas.DataFrame
    """
    option_margin_symbol_df = option_margin_symbol()
    url = option_margin_symbol_df[option_margin_symbol_df['symbol'] == symbol]['url'].values[0]
    r = requests.get(url)
    soup = BeautifulSoup(r.content, features="lxml")
    updated_time = soup.find_all("small")[0].get_text().strip("最近更新：")
    temp_df = pd.read_html(StringIO(r.text))[0]
    temp_df['更新时间'] = updated_time

    temp_df['结算价'] = pd.to_numeric(temp_df['结算价'], errors='coerce')
    temp_df['交易乘数'] = pd.to_numeric(temp_df['交易乘数'], errors='coerce')
    temp_df['买方权利金'] = pd.to_numeric(temp_df['买方权利金'], errors='coerce')
    temp_df['卖方保证金'] = pd.to_numeric(temp_df['卖方保证金'], errors='coerce')
    temp_df['开仓手续费'] = pd.to_numeric(temp_df['开仓手续费'], errors='coerce')
    temp_df['平今手续费'] = pd.to_numeric(temp_df['平今手续费'], errors='coerce')
    temp_df['平昨手续费'] = pd.to_numeric(temp_df['平昨手续费'], errors='coerce')
    temp_df['手续费(开+平今)'] = pd.to_numeric(temp_df['手续费(开+平今)'], errors='coerce')
    return temp_df


if __name__ == '__main__':
    option_margin_df = option_margin(symbol="原油期权")
    print(option_margin_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/7/24 23:30
Desc: 巨潮资讯-数据中心-专题统计-股东股本-实际控制人持股变动
https://webapi.cninfo.com.cn/#/thematicStatistics

巨潮资讯-数据中心-专题统计-股东股本-高管持股变动明细
https://webapi.cninfo.com.cn/#/thematicStatistics
"""

import datetime

import pandas as pd
import py_mini_racer
import requests

from akshare.datasets import get_ths_js


def _get_file_content_cninfo(file: str = "cninfo.js") -> str:
    """
    获取 JS 文件的内容
    :param file:  JS 文件名
    :type file: str
    :return: 文件内容
    :rtype: str
    """
    setting_file_path = get_ths_js(file)
    with open(setting_file_path, encoding="utf-8") as f:
        file_data = f.read()
    return file_data


def stock_hold_control_cninfo(symbol: str = "全部") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-股东股本-实际控制人持股变动
    https://webapi.cninfo.com.cn/#/thematicStatistics
    :param symbol: choice of {"单独控制", "实际控制人", "一致行动人", "家族控制", "全部"}; 从 2010 开始
    :type symbol: str
    :return: 实际控制人持股变动
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "单独控制": "069001",
        "实际控制人": "069002",
        "一致行动人": "069003",
        "家族控制": "069004",
        "全部": "",
    }
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1033"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "ctype": symbol_map[symbol],
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "控股比例",
        "控股数量",
        "证券简称",
        "实际控制人名称",
        "直接控制人名称",
        "控制类型",
        "证券代码",
        "变动日期",
    ]
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "变动日期",
            "实际控制人名称",
            "控股数量",
            "控股比例",
            "直接控制人名称",
            "控制类型",
        ]
    ]
    temp_df["变动日期"] = pd.to_datetime(temp_df["变动日期"], errors="coerce").dt.date
    temp_df["控股数量"] = pd.to_numeric(temp_df["控股数量"], errors="coerce")
    temp_df["控股比例"] = pd.to_numeric(temp_df["控股比例"], errors="coerce")
    return temp_df


def stock_hold_management_detail_cninfo(symbol: str = "增持") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-股东股本-高管持股变动明细
    https://webapi.cninfo.com.cn/#/thematicStatistics
    :param symbol: choice of {"增持", "减持"}
    :type symbol: str
    :return: 高管持股变动明细
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "增持": "B",
        "减持": "S",
    }
    current_date = datetime.datetime.now().date().isoformat()
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1030"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "*/*",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Accept-Enckey": mcode,
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "sdate": str(int(current_date[:4]) - 1) + current_date[4:],
        "edate": current_date,
        "varytype": symbol_map[symbol],
    }
    r = requests.post(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "证券简称",
        "公告日期",
        "高管姓名",
        "期末市值",
        "成交均价",
        "证券代码",
        "变动比例",
        "变动数量",
        "截止日期",
        "期末持股数量",
        "期初持股数量",
        "变动人与董监高关系",
        "董监高职务",
        "董监高姓名",
        "数据来源",
        "持股变动原因",
    ]
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "截止日期",
            "公告日期",
            "高管姓名",
            "董监高姓名",
            "董监高职务",
            "变动人与董监高关系",
            "期初持股数量",
            "期末持股数量",
            "变动数量",
            "变动比例",
            "成交均价",
            "期末市值",
            "持股变动原因",
            "数据来源",
        ]
    ]
    temp_df["截止日期"] = pd.to_datetime(temp_df["截止日期"], errors="coerce").dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["期初持股数量"] = pd.to_numeric(temp_df["期初持股数量"], errors="coerce")
    temp_df["期末持股数量"] = pd.to_numeric(temp_df["期末持股数量"], errors="coerce")
    temp_df["变动数量"] = pd.to_numeric(temp_df["变动数量"], errors="coerce")
    temp_df["变动比例"] = pd.to_numeric(temp_df["变动比例"], errors="coerce")
    temp_df["成交均价"] = pd.to_numeric(temp_df["成交均价"], errors="coerce")
    temp_df["期末市值"] = pd.to_numeric(temp_df["期末市值"], errors="coerce")
    return temp_df


def stock_hold_change_cninfo(symbol: str = "全部") -> pd.DataFrame:
    """
    巨潮资讯-数据中心-专题统计-股东股本-股本变动
    https://webapi.cninfo.com.cn/#/thematicStatistics
    :param symbol: choice of {"深市主板", "沪市", "创业板", "科创板", "北交所", "全部"}
    :type symbol: str
    :return: 股本变动
    :rtype: pandas.DataFrame
    """
    symbol_map = {
        "深市主板": "012002",
        "沪市": "012001",
        "创业板": "012015",
        "科创板": "012029",
        "北交所": "012046",
        "全部": "",
    }
    url = "https://webapi.cninfo.com.cn/api/sysapi/p_sysapi1029"
    js_code = py_mini_racer.MiniRacer()
    js_content = _get_file_content_cninfo("cninfo.js")
    js_code.eval(js_content)
    mcode = js_code.call("getResCode1")
    headers = {
        "Accept": "/",
        "Accept-Enckey": mcode,
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Cache-Control": "no-cache",
        "Content-Length": "0",
        "Host": "webapi.cninfo.com.cn",
        "Origin": "https://webapi.cninfo.com.cn",
        "Pragma": "no-cache",
        "Proxy-Connection": "keep-alive",
        "Referer": "https://webapi.cninfo.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/93.0.4577.63 Safari/537.36",
        "X-Requested-With": "XMLHttpRequest",
    }
    params = {
        "market": symbol_map[symbol],
    }
    r = requests.get(url, headers=headers, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["records"])
    temp_df.columns = [
        "已流通股份",
        "总股本",
        "交易市场",
        "证券简称",
        "公告日期",
        "变动原因",
        "证券代码",
        "变动日期",
        "流通受限股份",
        "已流通比例",
    ]
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "交易市场",
            "公告日期",
            "变动日期",
            "变动原因",
            "总股本",
            "已流通股份",
            "已流通比例",
            "流通受限股份",
        ]
    ]
    temp_df["变动日期"] = pd.to_datetime(temp_df["变动日期"], errors="coerce").dt.date
    temp_df["公告日期"] = pd.to_datetime(temp_df["公告日期"], errors="coerce").dt.date
    temp_df["总股本"] = pd.to_numeric(temp_df["总股本"], errors="coerce")
    temp_df["已流通股份"] = pd.to_numeric(temp_df["已流通股份"], errors="coerce")
    temp_df["已流通比例"] = pd.to_numeric(temp_df["已流通比例"], errors="coerce")
    temp_df["流通受限股份"] = pd.to_numeric(temp_df["流通受限股份"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_hold_control_cninfo_df = stock_hold_control_cninfo(symbol="全部")
    print(stock_hold_control_cninfo_df)

    stock_hold_management_detail_cninfo_df = stock_hold_management_detail_cninfo(
        symbol="增持"
    )
    print(stock_hold_management_detail_cninfo_df)

    stock_hold_change_cninfo_df = stock_hold_change_cninfo(symbol="全部")
    print(stock_hold_change_cninfo_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/10/30 17:40
Desc: openctp-合约信息接口
http://openctp.cn/instruments.html
"""

import pandas as pd
import requests


def option_contract_info_ctp() -> pd.DataFrame:
    """
    openctp-合约信息接口-期权合约
    http://openctp.cn/instruments.html
    :return: 期权合约信息
    :rtype: pandas.DataFrame
    """
    url = "http://dict.openctp.cn/instruments?types=option"
    r = requests.get(url)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['data'])

    # 字段映射：英文字段名 -> 中文字段名
    column_mapping = {
        'ExchangeID': '交易所ID',
        'InstrumentID': '合约ID',
        'InstrumentName': '合约名称',
        'ProductClass': '商品类别',
        'ProductID': '品种ID',
        'VolumeMultiple': '合约乘数',
        'PriceTick': '最小变动价位',
        'LongMarginRatioByMoney': '做多保证金率',
        'ShortMarginRatioByMoney': '做空保证金率',
        'LongMarginRatioByVolume': '做多保证金/手',
        'ShortMarginRatioByVolume': '做空保证金/手',
        'OpenRatioByMoney': '开仓手续费率',
        'OpenRatioByVolume': '开仓手续费/手',
        'CloseRatioByMoney': '平仓手续费率',
        'CloseRatioByVolume': '平仓手续费/手',
        'CloseTodayRatioByMoney': '平今手续费率',
        'CloseTodayRatioByVolume': '平今手续费/手',
        'DeliveryYear': '交割年份',
        'DeliveryMonth': '交割月份',
        'OpenDate': '上市日期',
        'ExpireDate': '最后交易日',
        'DeliveryDate': '交割日',
        'UnderlyingInstrID': '标的合约ID',
        'UnderlyingMultiple': '标的合约乘数',
        'OptionsType': '期权类型',
        'StrikePrice': '行权价',
        'InstLifePhase': '合约状态'
    }
    # 重命名列为中文
    temp_df = temp_df.rename(columns=column_mapping)
    return temp_df


# 使用示例
if __name__ == "__main__":
    option_contract_info_ctp_df = option_contract_info_ctp()
    print(option_contract_info_ctp_df)



# !/usr/bin/env python
"""
Date: 2025/3/10 18:00
Desc: 通用帮助函数
"""

import math
import random
import time
from typing import List, Dict

import pandas as pd

from akshare.utils.request import request_with_retry
from akshare.utils.tqdm import get_tqdm


def fetch_paginated_data(url: str, base_params: Dict, timeout: int = 15):
    """
    东方财富-分页获取数据并合并结果
    https://quote.eastmoney.com/f1.html?newcode=0.000001
    :param url: 股票代码
    :type url: str
    :param base_params: 基础请求参数
    :type base_params: dict
    :param timeout: 请求超时时间
    :type timeout: str
    :return: 合并后的数据
    :rtype: pandas.DataFrame
    """
    # 复制参数以避免修改原始参数
    params = base_params.copy()
    # 获取第一页数据，用于确定分页信息
    r = request_with_retry(url, params=params, timeout=timeout)
    data_json = r.json()
    # 计算分页信息
    per_page_num = len(data_json["data"]["diff"])
    total_page = math.ceil(data_json["data"]["total"] / per_page_num)
    # 存储所有页面数据
    temp_list = []
    # 添加第一页数据
    temp_list.append(pd.DataFrame(data_json["data"]["diff"]))
    # 获取进度条
    tqdm = get_tqdm()
    # 获取剩余页面数据
    for page in tqdm(range(2, total_page + 1), leave=False):
        params.update({"pn": page})
        # 添加随机延迟，避免请求过于频繁
        time.sleep(random.uniform(0.5, 1.5))
        r = request_with_retry(url, params=params, timeout=timeout)
        data_json = r.json()
        inner_temp_df = pd.DataFrame(data_json["data"]["diff"])
        temp_list.append(inner_temp_df)
    # 合并所有数据
    temp_df = pd.concat(temp_list, ignore_index=True)
    temp_df["f3"] = pd.to_numeric(temp_df["f3"], errors="coerce")
    temp_df.sort_values(by=["f3"], ascending=False, inplace=True, ignore_index=True)
    temp_df.reset_index(inplace=True)
    temp_df["index"] = temp_df["index"].astype(int) + 1
    return temp_df


def set_df_columns(df: pd.DataFrame, cols: List[str]) -> pd.DataFrame:
    """
    设置 pandas.DataFrame 为空的情况
    :param df: 需要设置命名的数据框
    :type df: pandas.DataFrame
    :param cols: 字段的列表
    :type cols: list
    :return: 重新设置后的数据
    :rtype: pandas.DataFrame
    """
    if df.shape == (0, 0):
        return pd.DataFrame(data=[], columns=cols)
    else:
        df.columns = cols
        return df



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2019/10/20 10:58
Desc: 外汇配置文件
"""

# headers
SHORT_HEADERS = {
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.91 Safari/537.36"
}
# url
FX_SPOT_URL = (
    "http://www.chinamoney.com.cn/r/cms/www/chinamoney/data/fx/rfx-sp-quot.json"
)
FX_SWAP_URL = (
    "http://www.chinamoney.com.cn/r/cms/www/chinamoney/data/fx/rfx-sw-quot.json"
)
FX_PAIR_URL = (
    "http://www.chinamoney.com.cn/r/cms/www/chinamoney/data/fx/cpair-quot.json"
)
# payload
SPOT_PAYLOAD = {"t": {}}



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/1/19 15:00
Desc: 深圳证券交易所-ETF基金份额数据
https://fund.szse.cn/marketdata/fundslist/index.html
"""

import warnings

import pandas as pd
import requests


def fund_etf_scale_szse() -> pd.DataFrame:
    """
    深圳证券交易所-基金产品-基金列表-ETF基金份额
    https://fund.szse.cn/marketdata/fundslist/index.html
    :return: ETF基金份额数据
    :rtype: pandas.DataFrame
    """
    url = "https://fund.szse.cn/api/report/ShowReport"
    params = {
        "SHOWTYPE": "xlsx",
        "CATALOGID": "1000_lf",
        "TABKEY": "tab1",
        "random": "0.07610353191740105",
    }
    headers = {
        "Referer": "https://fund.szse.cn/marketdata/fundslist/index.html",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/88.0.4324.150 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    with warnings.catch_warnings(record=True):
        warnings.simplefilter("always")
        temp_df = pd.read_excel(r.content, engine="openpyxl", dtype={"基金代码": str})
    temp_df.rename(
        columns={
            "当前规模(份)": "基金份额",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "基金代码",
            "基金简称",
            "基金类别",
            "投资类别",
            "上市日期",
            "基金份额",
            "基金管理人",
            "基金发起人",
            "基金托管人",
            "净值",
        ]
    ]
    temp_df["上市日期"] = pd.to_datetime(temp_df["上市日期"], errors="coerce").dt.date
    temp_df["基金份额"] = pd.to_numeric(temp_df["基金份额"], errors="coerce")
    temp_df["净值"] = pd.to_numeric(temp_df["净值"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    fund_etf_scale_szse_df = fund_etf_scale_szse()
    print(fund_etf_scale_szse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2026/1/13 15:20
Desc: 上海证券交易所-融资融券数据
https://www.sse.com.cn/market/othersdata/margin/sum/
"""

import pandas as pd
import requests


def stock_margin_ratio_pa(symbol: str = "深市", date: str = "20260113") -> pd.DataFrame:
    """
    融资融券-标的证券名单及保证金比例查询
    https://stock.pingan.com/static/webinfo/margin/business.html?businessType=0
    :param symbol: choice of {"深市", "沪市", "北交所"}
    :type symbol: str
    :param date: 交易日期
    :type date: str
    :return: 标的证券名单及保证金比例查询
    :rtype: pandas.DataFrame
    """
    market_code = {
        "深市": "00",
        "沪市": "10",
        "北交所": "30",
    }
    url = "https://stock.pingan.com/fss/servlet/fsscoreapp/stockSource/mrgRatio"
    payload = {
        "currentPage": 1,
        "pageSize": 50000,
        "type": "bdzq",
        "setdate": "-".join([date[:4], date[4:6], date[6:]]),
        "stockMes": "",
        "market": market_code[symbol],
        "appName": "AYLCH5",
        "tokenId": "",
        "appChannel": "LRSP",
        "requestId": "194055910e2075c03e25fabf6ffc5a7f",
        "channel": "pa18",
    }
    r = requests.post(url, json=payload)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["data"]["list"])
    temp_df.rename(
        columns={
            "fiMarginRatio": "融资比例",
            "secuCode": "证券代码",
            "secuName": "证券简称",
            "slMarginRatio": "融券比例",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "证券代码",
            "证券简称",
            "融资比例",
            "融券比例",
        ]
    ]
    temp_df["融资比例"] = pd.to_numeric(temp_df["融资比例"], errors="coerce")
    temp_df["融券比例"] = pd.to_numeric(temp_df["融券比例"], errors="coerce")
    return temp_df


def stock_margin_sse(
    start_date: str = "20010106", end_date: str = "20230922"
) -> pd.DataFrame:
    """
    上海证券交易所-融资融券数据-融资融券汇总
    https://www.sse.com.cn/market/othersdata/margin/sum/
    :param start_date: 交易开始日期
    :type start_date: str
    :param end_date: 交易结束日期
    :type end_date: str
    :return: 融资融券汇总
    :rtype: pandas.DataFrame
    """
    url = "https://query.sse.com.cn/marketdata/tradedata/queryMargin.do"
    params = {
        "isPagination": "true",
        "beginDate": start_date,
        "endDate": end_date,
        "tabType": "",
        "stockCode": "",
        "pageHelp.pageSize": "5000",
        "pageHelp.pageNo": "1",
        "pageHelp.beginPage": "1",
        "pageHelp.cacheSize": "1",
        "pageHelp.endPage": "5",
    }
    headers = {
        "Referer": "https://www.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/88.0.4324.150 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    temp_df.columns = [
        "_",
        "信用交易日期",
        "_",
        "融券卖出量",
        "融券余量",
        "融券余量金额",
        "_",
        "_",
        "融资买入额",
        "融资融券余额",
        "融资余额",
        "_",
        "_",
    ]
    temp_df = temp_df[
        [
            "信用交易日期",
            "融资余额",
            "融资买入额",
            "融券余量",
            "融券余量金额",
            "融券卖出量",
            "融资融券余额",
        ]
    ]
    temp_df["融资余额"] = pd.to_numeric(temp_df["融资余额"], errors="coerce")
    temp_df["融资买入额"] = pd.to_numeric(temp_df["融资买入额"], errors="coerce")
    temp_df["融券余量"] = pd.to_numeric(temp_df["融券余量"], errors="coerce")
    temp_df["融券余量金额"] = pd.to_numeric(temp_df["融券余量金额"], errors="coerce")
    temp_df["融券卖出量"] = pd.to_numeric(temp_df["融券卖出量"], errors="coerce")
    temp_df["融资融券余额"] = pd.to_numeric(temp_df["融资融券余额"], errors="coerce")
    return temp_df


def stock_margin_detail_sse(date: str = "20230922") -> pd.DataFrame:
    """
    上海证券交易所-融资融券数据-融资融券明细
    https://www.sse.com.cn/market/othersdata/margin/detail/
    :param date: 交易日期
    :type date: str
    :return: 融资融券明细
    :rtype: pandas.DataFrame
    """
    url = "https://query.sse.com.cn/marketdata/tradedata/queryMargin.do"
    params = {
        "isPagination": "true",
        "tabType": "mxtype",
        "detailsDate": date,
        "stockCode": "",
        "beginDate": "",
        "endDate": "",
        "pageHelp.pageSize": "5000",
        "pageHelp.pageCount": "50",
        "pageHelp.pageNo": "1",
        "pageHelp.beginPage": "1",
        "pageHelp.cacheSize": "1",
        "pageHelp.endPage": "21",
    }
    headers = {
        "Referer": "https://www.sse.com.cn/",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
                      "Chrome/88.0.4324.150 Safari/537.36",
    }
    r = requests.get(url, params=params, headers=headers)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"])
    temp_df.columns = [
        "_",
        "信用交易日期",
        "融券偿还量",
        "融券卖出量",
        "融券余量",
        "_",
        "_",
        "融资偿还额",
        "融资买入额",
        "_",
        "融资余额",
        "标的证券简称",
        "标的证券代码",
    ]
    temp_df = temp_df[
        [
            "信用交易日期",
            "标的证券代码",
            "标的证券简称",
            "融资余额",
            "融资买入额",
            "融资偿还额",
            "融券余量",
            "融券卖出量",
            "融券偿还量",
        ]
    ]
    temp_df["融资余额"] = pd.to_numeric(temp_df["融资余额"], errors="coerce")
    temp_df["融资买入额"] = pd.to_numeric(temp_df["融资买入额"], errors="coerce")
    temp_df["融资偿还额"] = pd.to_numeric(temp_df["融资偿还额"], errors="coerce")
    temp_df["融券余量"] = pd.to_numeric(temp_df["融券余量"], errors="coerce")
    temp_df["融券卖出量"] = pd.to_numeric(temp_df["融券卖出量"], errors="coerce")
    temp_df["融券偿还量"] = pd.to_numeric(temp_df["融券偿还量"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    stock_margin_ratio_pa_df = stock_margin_ratio_pa(symbol="沪市", date="20260113")
    print(stock_margin_ratio_pa_df)

    stock_margin_sse_df = stock_margin_sse(start_date="20010106", end_date="20210401")
    print(stock_margin_sse_df)

    stock_margin_detail_sse_df = stock_margin_detail_sse(date="20230922")
    print(stock_margin_detail_sse_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/9/24 15:00
Desc: 东方财富-行情中心-同行比较
https://emweb.securities.eastmoney.com/pc_hsf10/pages/index.html?type=web&code=000895&color=b#/thbj
"""

import pandas as pd
import requests


def stock_zh_growth_comparison_em(symbol: str = "SZ000895") -> pd.DataFrame:
    """
    东方财富-行情中心-同行比较-成长性比较
    https://emweb.securities.eastmoney.com/pc_hsf10/pages/index.html?type=web&code=000895&color=b#/thbj/czxbj
    :param symbol: 股票代码
    :type symbol: str
    :return: 成长性比较
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_GROWTH',
        'columns': 'ALL',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol[2:]}.{symbol[:2]}")',
        'pageNumber': '',
        'pageSize': '',
        'sortTypes': '1',
        'sortColumns': 'PAIMING',
        'source': 'HSF10',
        'client': 'PC',
        'v': '02747607708067783'
    }
    r = requests.get(url, params=params)
    data_json = r.json()

    field_mapping = {
        'CORRE_SECURITY_CODE': '代码',
        'CORRE_SECURITY_NAME': '简称',
        'MGSY_3Y': '基本每股收益增长率-3年复合',
        'MGSYTB': '基本每股收益增长率-24A',
        'MGSYTTM': '基本每股收益增长率-TTM',
        'MGSY_1E': '基本每股收益增长率-25E',
        'MGSY_2E': '基本每股收益增长率-26E',
        'MGSY_3E': '基本每股收益增长率-27E',
        'YYSR_3Y': '营业收入增长率-3年复合',
        'YYSRTB': '营业收入增长率-24A',
        'YYSRTTM': '营业收入增长率-TTM',
        'YYSR_1E': '营业收入增长率-25E',
        'YYSR_2E': '营业收入增长率-26E',
        'YYSR_3E': '营业收入增长率-27E',
        'JLR_3Y': '净利润增长率-3年复合',
        'JLRTB': '净利润增长率-24A',
        'JLRTTM': '净利润增长率-TTM',
        'JLR_1E': '净利润增长率-25E',
        'JLR_2E': '净利润增长率-26E',
        'JLR_3E': '净利润增长率-27E',
        'PAIMING': '基本每股收益增长率-3年复合排名',
    }

    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json['result'] is not None:
        temp_df = pd.DataFrame(data_json['result']['data'])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]

    return temp_df


def stock_zh_valuation_comparison_em(symbol: str = "SZ000895") -> pd.DataFrame:
    """
    东方财富-行情中心-同行比较-估值比较
    https://emweb.securities.eastmoney.com/pc_hsf10/pages/index.html?type=web&code=000895&color=b#/thbj/gzbj
    :param symbol: 股票代码
    :type symbol: str
    :return: 估值比较
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_CVALUE',
        'columns': 'ALL',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol[2:]}.{symbol[:2]}")',
        'pageNumber': '',
        'pageSize': '',
        'sortTypes': '1',
        'sortColumns': 'PAIMING',
        'source': 'HSF10',
        'client': 'PC',
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json['result']['data'])
    field_mapping_result = {
        "CORRE_SECUCODE": "对应证券代码",
        "CORRE_SECURITY_CODE": "代码",
        "CORRE_SECURITY_NAME": "简称",
        "PB": "市净率-24A",
        "PB_MRQ": "市净率-MRQ",
        "PCE": "市现率1-24A",
        "PCE_TTM": "市现率1-TTM",
        "PCF": "市现率2-24A",
        "PCF_TTM": "市现率2-TTM",
        "PEG": "PEG",
        "PE_1Y": "市盈率-25E",
        "PE_2Y": "市盈率-26E",
        "PE_3Y": "市盈率-27E",
        "PE_TTM": "市盈率-TTM",
        "PS": "市销率-24A",
        "PS_1Y": "市销率-25E",
        "PS_2Y": "市销率-26E",
        "PS_3Y": "市销率-27E",
        "PS_TTM": "市销率-TTM",
        "QYBS": "EV/EBITDA-24A",
        "REPORT_DATE": "报告日期",
        "SECUCODE": "证券代码",
        "SECURITY_CODE": "行业标识",
        "TOTAL_COUNT": "证券数量",
        "PAIMING": "排名"
    }
    temp_df.rename(columns=field_mapping_result, inplace=True)
    temp_df = temp_df[[
        "排名",
        "代码",
        "简称",
        "PEG",
        "市盈率-TTM",
        "市盈率-25E",
        "市盈率-26E",
        "市盈率-27E",
        "市销率-24A",
        "市销率-TTM",
        "市销率-25E",
        "市销率-26E",
        "市销率-27E",
        "市净率-24A",
        "市净率-MRQ",
        "市现率1-24A",
        "市现率1-TTM",
        "市现率2-24A",
        "市现率2-TTM",
        "EV/EBITDA-24A",
    ]]
    temp_df = pd.concat([temp_df.iloc[-1:], temp_df.iloc[:-1]]).reset_index(drop=True)
    temp_df['排名'] = temp_df['排名'].astype(str)
    temp_df.iloc[0, 0] = f"{temp_df.iloc[0, 0]}/{data_json['result']['data'][0]['TOTAL_COUNT']}"
    row1 = temp_df.iloc[1].copy()
    row2 = temp_df.iloc[2].copy()
    # 交换位置
    temp_df.iloc[1] = row2
    temp_df.iloc[2] = row1
    return temp_df


def stock_zh_dupont_comparison_em(symbol: str = "SZ000895") -> pd.DataFrame:
    """
    东方财富-行情中心-同行比较-杜邦分析比较
    https://emweb.securities.eastmoney.com/pc_hsf10/pages/index.html?type=web&code=000895&color=b#/thbj/dbfxbj
    :param symbol: 股票代码
    :type symbol: str
    :return: 杜邦分析比较
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_DBFX',
        'columns': 'ALL',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol[2:]}.{symbol[:2]}")',
        'pageNumber': '',
        'pageSize': '',
        'sortTypes': '1',
        'sortColumns': 'PAIMING',
        'source': 'HSF10',
        'client': 'PC',
        'v': '05086361194054821'
    }
    r = requests.get(url, params=params)
    data_json = r.json()

    field_mapping = {
        'CORRE_SECURITY_CODE': '代码',
        'CORRE_SECURITY_NAME': '简称',
        'ROE_AVG': 'ROE-3年平均',
        'ROEPJ_L3': 'ROE-22A',
        'ROEPJ_L2': 'ROE-23A',
        'ROEPJ_L1': 'ROE-24A',
        'XSJLL_AVG': '净利率-3年平均',
        'XSJLL_L3': '净利率-22A',
        'XSJLL_L2': '净利率-23A',
        'XSJLL_L1': '净利率-24A',
        'TOAZZL_AVG': '总资产周转率-3年平均',
        'TOAZZL_L3': '总资产周转率-22A',
        'TOAZZL_L2': '总资产周转率-23A',
        'TOAZZL_L1': '总资产周转率-24A',
        'QYCS_AVG': '权益乘数-3年平均',
        'QYCS_L3': '权益乘数-22A',
        'QYCS_L2': '权益乘数-23A',
        'QYCS_L1': '权益乘数-24A',
        'PAIMING': 'ROE-3年平均排名',
    }

    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json['result'] is not None:
        temp_df = pd.DataFrame(data_json['result']['data'])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]

    return temp_df


def stock_zh_scale_comparison_em(symbol: str = "SZ000895") -> pd.DataFrame:
    """
    东方财富-行情中心-同行比较-公司规模
    https://emweb.securities.eastmoney.com/pc_hsf10/pages/index.html?type=web&code=000895&color=b#/thbj/gsgm
    :type symbol: str
    :return: 公司规模
    :rtype: pandas.DataFrame
    """
    url = 'https://datacenter.eastmoney.com/securities/api/data/v1/get'
    params = {
        'reportName': 'RPT_PCF10_INDUSTRY_MARKET',
        'columns': 'SECUCODE,SECURITY_CODE,SECURITY_NAME_ABBR,ORG_CODE,'
                   'CORRE_SECUCODE,CORRE_SECURITY_CODE,CORRE_SECURITY_NAME,'
                   'CORRE_ORG_CODE,TOTAL_CAP,FREECAP,TOTAL_OPERATEINCOME,'
                   'NETPROFIT,REPORT_TYPE,TOTAL_CAP_RANK,FREECAP_RANK,'
                   'TOTAL_OPERATEINCOME_RANK,NETPROFIT_RANK',
        'quoteColumns': '',
        'filter': f'(SECUCODE="{symbol[2:]}.{symbol[:2]}")(CORRE_SECUCODE="{symbol[2:]}.{symbol[:2]}")',
        'pageNumber': '1',
        'pageSize': '5',
        'sortTypes': '-1',
        'sortColumns': 'TOTAL_CAP',
        'source': 'HSF10',
        'client': 'PC',
        'v': '005391946600478148'
    }
    r = requests.get(url, params=params)
    data_json = r.json()

    field_mapping = {
        'CORRE_SECURITY_CODE': '代码',
        'CORRE_SECURITY_NAME': '简称',
        'TOTAL_CAP': '总市值',
        'TOTAL_CAP_RANK': '总市值排名',
        'FREECAP': '流通市值',
        'FREECAP_RANK': '流通市值排名',
        'TOTAL_OPERATEINCOME': '营业收入',
        'TOTAL_OPERATEINCOME_RANK': '营业收入排名',
        'NETPROFIT': '净利润',
        'NETPROFIT_RANK': '净利润排名',
    }

    temp_df = pd.DataFrame(columns=field_mapping.values())
    if data_json['result'] is not None:
        temp_df = pd.DataFrame(data_json['result']['data'])
        temp_df.rename(columns=field_mapping, inplace=True)
        temp_df = temp_df[field_mapping.values()]

    return temp_df


if __name__ == "__main__":
    stock_zh_growth_comparison_em_df = stock_zh_growth_comparison_em(symbol="SZ000895")
    print(stock_zh_growth_comparison_em_df)

    stock_zh_valuation_comparison_em_df = stock_zh_valuation_comparison_em(symbol="SZ000895")
    print(stock_zh_valuation_comparison_em_df)

    stock_zh_dupont_comparison_em_df = stock_zh_dupont_comparison_em(symbol="SZ000895")
    print(stock_zh_dupont_comparison_em_df)

    stock_zh_scale_comparison_em_df = stock_zh_scale_comparison_em(symbol="SZ000895")
    print(stock_zh_scale_comparison_em_df)
